@@ -1,6 +1,6 @@
 /* Misc util functions */
 
-char const *cvs_id_util = "$Id: util.c,v 1.8 2004/12/16 08:49:53 bodea Exp $";
+char const *cvs_id_util = "$Id: util.c,v 1.13 2005/09/19 00:29:12 bodea Exp $";
 
 #include <unistd.h>
 #include <errno.h>
@@ -39,7 +39,8 @@ void *shared_malloc(unsigned int size)
 	return p;
 }
 
-extern int udpfd, controlfd, tunfd, snoopfd, ifrfd, cluster_sockfd;
+extern int forked;
+extern int cluster_sockfd, tunfd, udpfd, controlfd, daefd, snoopfd, ifrfd, ifr6fd, rand_fd;
 extern int *radfds;
 
 pid_t fork_and_close()
@@ -50,6 +51,7 @@ pid_t fork_and_close()
 	if (pid)
 		return pid;
 
+	forked++;
 	if (config->scheduler_fifo)
 	{
 		struct sched_param params = {0};
@@ -63,23 +65,26 @@ pid_t fork_and_close()
 
 	signal(SIGPIPE, SIG_DFL);
 	signal(SIGCHLD, SIG_DFL);
-	signal(SIGHUP, SIG_DFL);
+	signal(SIGHUP,  SIG_DFL);
 	signal(SIGUSR1, SIG_DFL);
 	signal(SIGQUIT, SIG_DFL);
 	signal(SIGKILL, SIG_DFL);
-	signal(SIGALRM, SIG_DFL);
 	signal(SIGTERM, SIG_DFL);
 
 	// Close sockets
+	if (clifd != -1)          close(clifd);
+	if (cluster_sockfd != -1) close(cluster_sockfd);
 	if (tunfd != -1)          close(tunfd);
 	if (udpfd != -1)          close(udpfd);
 	if (controlfd != -1)      close(controlfd);
+	if (daefd != -1)          close(daefd);
 	if (snoopfd != -1)        close(snoopfd);
 	if (ifrfd != -1)          close(ifrfd);
-	if (cluster_sockfd != -1) close(cluster_sockfd);
-	if (clifd != -1)          close(clifd);
+	if (ifr6fd != -1)         close(ifr6fd);
+	if (rand_fd != -1)        close(rand_fd);
+	if (epollfd != -1)        close(epollfd);
 
-	for (i = 0; radfds && i < config->num_radfds; i++)
+	for (i = 0; radfds && i < RADIUS_FDS; i++)
 		close(radfds[i]);
 #ifdef BGP
 	for (i = 0; i < BGP_NUM_PEERS; i++)
